@@ -4,9 +4,9 @@
     We use environment variables for controlling QCheck here, since alcotest
     doesn't seem to provide a lot of flexibility.
 
-    [QCHECK_VERBOSE] if "1" or "true", will make tests verbose
-    [QCHECK_SEED] if an integer, will fix the seed
-    [QCHECK_LONG] is present, will trigger long tests
+    - [QCHECK_VERBOSE] if "1" or "true", will make tests verbose
+    - [QCHECK_SEED] if an integer, will fix the seed
+    - [QCHECK_LONG] is present, will trigger long tests
 
     @since 0.9
 *)
