@@ -94,6 +94,10 @@ module Gen = struct
 
   let frequency l st = frequencyl l st st
 
+  let small_nat st =
+    let p = RS.float st 1. in
+    if p < 0.75 then RS.int st 10 else RS.int st 100
+
   (* natural number generator *)
   let nat st =
     let p = RS.float st 1. in
@@ -102,7 +106,10 @@ module Gen = struct
     else if p < 0.95 then RS.int st 1_000
     else RS.int st 10_000
 
-  let small_nat = nat
+  let big_nat st =
+    let p = RS.float st 1. in
+    if p < 0.75 then nat st
+    else RS.int st 1_000_000
 
   let unit _st = ()
 
@@ -115,6 +122,20 @@ module Gen = struct
   let pfloat st = abs_float (float st)
   let nfloat st = -.(pfloat st)
 
+  let float_bound_inclusive bound st = RS.float st bound
+
+  let float_bound_exclusive bound st =
+    match bound with
+    | 0. -> raise (Invalid_argument "Gen.float_bound_exclusive")
+    | b_pos when bound > 0. -> RS.float st (b_pos -. epsilon_float)
+    | b_neg -> RS.float st (b_neg +. epsilon_float)
+
+  let float_range low high =
+    if high < low || high -. low > max_float then invalid_arg "Gen.float_range";
+    fun st -> low +. (float_bound_inclusive (high -. low) st)
+
+  let (--.) = float_range
+
   let neg_int st = -(nat st)
 
   let opt f st =
@@ -135,9 +156,9 @@ module Gen = struct
   let int st = if RS.bool st then - (pint st) - 1 else pint st
   let int_bound n =
     if n < 0 then invalid_arg "Gen.int_bound";
-    fun st ->
-      let r = pint st in
-      r mod (n+1)
+    if n <= (1 lsl 30) - 2
+    then fun st -> Random.State.int st (n + 1)
+    else fun st -> let r = pint st in r mod (n + 1)
   let int_range a b =
     if b < a then invalid_arg "Gen.int_range";
     fun st -> a + (int_bound (b-a) st)
@@ -188,6 +209,14 @@ module Gen = struct
     shuffle_a a st;
     Array.to_list a
 
+  let shuffle_w_l l st =
+    let sample (w, v) =
+      let fl_w = float_of_int w in
+      (float_bound_inclusive 1. st ** (1. /. fl_w), v)
+    in
+    let samples = List.rev_map sample l in
+    List.sort (fun (w1, _) (w2, _) -> compare w1 w2) samples |> List.rev_map snd
+
   let pair g1 g2 st = (g1 st, g2 st)
 
   let triple g1 g2 g3 st = (g1 st, g2 st, g3 st)
@@ -214,9 +243,12 @@ module Gen = struct
       Bytes.set s i (gen st)
     done;
     Bytes.unsafe_to_string s
-  let string ?gen st = string_size ?gen small_nat st
-  let small_string ?gen st = string_size ?gen (0--10) st
-  let small_list gen = list_size (0--10) gen
+  let string ?gen st = string_size ?gen nat st
+  let string_of gen = string_size ~gen nat
+  let string_readable = string_size ~gen:char nat
+  let small_string ?gen st = string_size ?gen small_nat st
+  let small_list gen = list_size small_nat gen
+  let small_array gen = array_size small_nat gen
 
   let join g st = (g st) st
 
@@ -325,22 +357,23 @@ module Shrink = struct
   type 'a t = 'a -> 'a Iter.t
 
   let nil _ = Iter.empty
+
   let unit = nil
 
   (* balanced shrinker for integers (non-exhaustive) *)
   let int x yield =
     let y = ref x in
     (* try some divisors *)
-    while !y < -2 || !y >2 do y := !y / 2; yield !y; done; (* fast path *)
+    while !y < -2 || !y >2 do y := !y / 2; yield (x - !y); done; (* fast path *)
     if x>0 then yield (x-1);
     if x<0 then yield (x+1);
     ()
 
   (* aggressive shrinker for integers,
-     get closer to 0, by dichotomy or just enumerating smaller values *)
+     get from 0 to x, by dichotomy or just enumerating smaller values *)
   let int_aggressive x yield =
     let y = ref x in
-    while !y < -2 || !y >2 do y := !y / 2; yield !y; done; (* fast path *)
+    while !y < -2 || !y >2 do y := !y / 2; yield (x - !y); done; (* fast path *)
     if x>0 then for i=x-1 downto 0 do yield i done;
     if x<0 then for i=x+1 to 0 do yield i done
 
@@ -386,9 +419,53 @@ module Shrink = struct
           )
         done
 
+  let list_spine l yield =
+    let n = List.length l in
+    let chunk_size = ref (n/2) in
+
+    (* push the [n] first elements of [l] into [q], return the rest of the list *)
+    let rec fill_queue n l q = match n,l with
+      | 0, _ -> l
+      | _, x::xs ->
+        Queue.push x q;
+        fill_queue (n-1) xs q
+      | _, _ -> assert false
+    in
+
+    (* remove elements from the list, by chunks of size [chunk_size] (bigger
+       chunks first) *)
+    while !chunk_size > 0 do
+      let q = Queue.create () in
+      let l' = fill_queue !chunk_size l q in
+      (* remove [chunk_size] elements in queue *)
+      let rec pos_loop rev_prefix suffix =
+        yield (List.rev_append rev_prefix suffix);
+        match suffix with
+        | [] -> ()
+        | x::xs ->
+          Queue.push x q;
+          let y = Queue.pop q in
+          (pos_loop [@tailcall]) (y::rev_prefix) xs
+      in
+      pos_loop [] l';
+      chunk_size := !chunk_size / 2;
+    done
+
+  let list_elems shrink l yield =
+    (* try to shrink each element of the list *)
+    let rec elem_loop rev_prefix suffix = match suffix with
+      | [] -> ()
+      | x::xs ->
+         shrink x (fun x' -> yield (List.rev_append rev_prefix (x'::xs)));
+         elem_loop (x::rev_prefix) xs
+    in
+    elem_loop [] l
+
   let list ?shrink l yield =
-    array ?shrink (Array.of_list l)
-      (fun a -> yield (Array.to_list a))
+    list_spine l yield;
+    match shrink with
+    | None -> ()
+    | Some shrink -> list_elems shrink l yield
 
   let pair a b (x,y) yield =
     a x (fun x' -> yield (x',y));
@@ -561,6 +638,14 @@ let float = make_scalar ~print:string_of_float Gen.float
 let pos_float = make_scalar ~print:string_of_float Gen.pfloat
 let neg_float = make_scalar ~print:string_of_float Gen.nfloat
 
+let float_bound_inclusive bound =
+  make_scalar ~print:string_of_float (Gen.float_bound_inclusive bound)
+
+let float_bound_exclusive bound =
+  make_scalar ~print:string_of_float (Gen.float_bound_exclusive bound)
+
+let float_range low high = make_scalar ~print:string_of_float (Gen.float_range low high)
+
 let int = make_int Gen.int
 let int_bound n = make_int (Gen.int_bound n)
 let int_range a b = make_int (Gen.int_range a b)
@@ -588,11 +673,11 @@ let string_gen gen =
 
 let string = string_gen Gen.char
 let string_of_size size = string_gen_of_size size Gen.char
-let small_string = string_gen_of_size Gen.(0--10) Gen.char
+let small_string = string_gen_of_size Gen.small_nat Gen.char
 
 let printable_string = string_gen Gen.printable
 let printable_string_of_size size = string_gen_of_size size Gen.printable
-let small_printable_string = string_gen_of_size Gen.(0--10) Gen.printable
+let small_printable_string = string_gen_of_size Gen.small_nat Gen.printable
 
 let numeral_string = string_gen Gen.numeral
 let numeral_string_of_size size = string_gen_of_size size Gen.numeral
@@ -1042,11 +1127,20 @@ module TestResult = struct
 
   type 'a failed_state = 'a counter_ex list
 
+  (** Result state.
+      changed in 0.10 (move to inline records) *)
   type 'a state =
     | Success
-    | Failed of 'a failed_state (** Failed instances *)
-    | Error of 'a counter_ex * exn * string (** Error, backtrace, and instance
-                                                that triggered it *)
+    | Failed of {
+        instances: 'a failed_state; (** Failed instance(s) *)
+      }
+    | Failed_other of {msg: string}
+    | Error of {
+        instance: 'a counter_ex;
+        exn: exn;
+        backtrace: string;
+      } (** Error, backtrace, and instance that triggered it *)
+
 
   (* result returned by running a test *)
   type 'a t = {
@@ -1055,6 +1149,7 @@ module TestResult = struct
     mutable count_gen: int; (* number of generated cases *)
     collect_tbl: (string, int) Hashtbl.t lazy_t;
     stats_tbl: ('a stat * (int, int) Hashtbl.t) list;
+    mutable warnings: string list;
     mutable instances: 'a list;
   }
 
@@ -1062,37 +1157,38 @@ module TestResult = struct
   let fail ~msg_l ~small ~steps:shrink_steps res instance =
     let c_ex = {instance; shrink_steps; msg_l; } in
     match res.state with
-    | Success -> res.state <- Failed [ c_ex ]
-    | Error (x, e, bt) ->
-        res.state <- Error (x,e,bt); (* same *)
-    | Failed [] -> assert false
-    | Failed (c_ex' :: _ as l) ->
+    | Success -> res.state <- Failed {instances=[ c_ex ]}
+    | Error _
+    | Failed_other _ -> ()
+    | Failed {instances=[]} -> assert false
+    | Failed {instances=((c_ex'::_) as l)} ->
         match small with
         | Some small ->
             (* all counter-examples in [l] have same size according to [small],
                so we just compare to the first one, and we enforce
                the invariant *)
             begin match Pervasives.compare (small instance) (small c_ex'.instance) with
-            | 0 -> res.state <- Failed (c_ex :: l) (* same size: add [c_ex] to [l] *)
-            | n when n<0 -> res.state <- Failed [c_ex] (* drop [l] *)
+            | 0 -> res.state <- Failed {instances=c_ex :: l} (* same size: add [c_ex] to [l] *)
+            | n when n<0 -> res.state <- Failed {instances=[c_ex]} (* drop [l] *)
             | _ -> () (* drop [c_ex], not small enough *)
             end
         | _ ->
             (* no [small] function, keep all counter-examples *)
             res.state <-
-              Failed (c_ex :: l)
+              Failed {instances=c_ex :: l}
 
-  let error ~msg_l ~steps res instance e bt =
-    res.state <- Error ({instance; shrink_steps=steps; msg_l; }, e, bt)
+  let error ~msg_l ~steps res instance exn backtrace =
+    res.state <- Error {instance={instance; shrink_steps=steps; msg_l; }; exn; backtrace}
 
   let collect r =
     if Lazy.is_val r.collect_tbl then Some (Lazy.force r.collect_tbl) else None
 
   let stats r = r.stats_tbl
+  let warnings r = r.warnings
 
   let is_success r = match r.state with
     | Success -> true
-    | Failed _ | Error _ -> false
+    | Failed _ | Error _ | Failed_other _ -> false
 end
 
 module Test = struct
@@ -1103,6 +1199,7 @@ module Test = struct
     max_fail : int; (* max number of failures *)
     law : 'a -> bool; (* the law to check *)
     arb : 'a arbitrary; (* how to generate/print/shrink instances *)
+    if_assumptions_fail: [`Fatal | `Warning] * float;
     mutable name : string; (* name of the law *)
   }
 
@@ -1122,7 +1219,10 @@ module Test = struct
     let r = ref 0 in
     (fun () -> incr r; Printf.sprintf "anon_test_%d" !r)
 
-  let make_cell ?(count=default_count) ?(long_factor=1) ?max_gen
+  let default_if_assumptions_fail = `Warning, 0.05
+
+  let make_cell ?(if_assumptions_fail=default_if_assumptions_fail)
+      ?(count=default_count) ?(long_factor=1) ?max_gen
   ?(max_fail=1) ?small ?(name=fresh_name()) arb law
   =
     let max_gen = match max_gen with None -> count + 200 | Some x->x in
@@ -1135,10 +1235,11 @@ module Test = struct
       name;
       count;
       long_factor;
+      if_assumptions_fail;
     }
 
-  let make ?count ?long_factor ?max_gen ?max_fail ?small ?name arb law =
-    Test (make_cell ?count ?long_factor ?max_gen ?max_fail ?small ?name arb law)
+  let make ?if_assumptions_fail ?count ?long_factor ?max_gen ?max_fail ?small ?name arb law =
+    Test (make_cell ?if_assumptions_fail ?count ?long_factor ?max_gen ?max_fail ?small ?name arb law)
 
   (** {6 Running the test} *)
 
@@ -1175,7 +1276,7 @@ module Test = struct
     handler : 'a handler;
     rand: Random.State.t;
     mutable res: 'a TestResult.t;
-    mutable cur_count: int;  (** number of iterations to do *)
+    mutable cur_count: int;  (** number of iterations remaining to do *)
     mutable cur_max_gen: int; (** maximum number of generations allowed *)
     mutable cur_max_fail: int; (** maximum number of counter-examples allowed *)
   }
@@ -1339,25 +1440,53 @@ module Test = struct
 
   let callback_nil_ : _ callback = fun _ _ _ -> ()
 
+  (* check that there are sufficiently many tests which passed, to avoid
+     the case where they all passed by failed precondition *)
+  let check_if_assumptions target_count cell res : unit =
+    let percentage_of_count = float_of_int res.R.count /. float_of_int target_count in
+    let assm_flag, assm_frac = cell.if_assumptions_fail in
+    if R.is_success res && percentage_of_count < assm_frac then (
+      let msg =
+        format_of_string "%s: \
+         only %.1f%% tests (of %d) passed precondition for %S\n\n\
+         NOTE: it is likely that the precondition is too strong, or that \
+         the generator is buggy.\n%!"
+      in
+      match assm_flag with
+      | `Warning ->
+        let msg = Printf.sprintf
+          msg "WARNING"
+          (percentage_of_count *. 100.) cell.count cell.name in
+        res.R.warnings <- msg :: res.R.warnings
+      | `Fatal ->
+        (* turn it into an error *)
+        let msg = Printf.sprintf
+          msg "ERROR"
+          (percentage_of_count *. 100.) cell.count cell.name in
+        res.R.state <- R.Failed_other {msg}
+    )
+
   (* main checking function *)
   let check_cell ?(long=false) ?(call=callback_nil_)
       ?(step=step_nil_) ?(handler=handler_nil_)
       ?(rand=Random.State.make [| 0 |]) cell =
     let factor = if long then cell.long_factor else 1 in
+    let target_count = factor*cell.count in
     let state = {
       test=cell; rand;
       step; handler;
-      cur_count=factor*cell.count;
+      cur_count=target_count;
       cur_max_gen=factor*cell.max_gen;
       cur_max_fail=factor*cell.max_fail;
       res = {R.
         state=R.Success; count=0; count_gen=0;
         collect_tbl=lazy (Hashtbl.create 10);
-        instances=[];
+        instances=[]; warnings=[];
         stats_tbl= List.map (fun stat -> stat, Hashtbl.create 10) cell.arb.stats;
       };
     } in
     let res = check_state state in
+    check_if_assumptions target_count cell res;
     call cell.name cell res;
     res
 
@@ -1454,10 +1583,11 @@ module Test = struct
     let hist_size, bucket_size =
       let sample_width = Int64.(sub (of_int max_idx) (of_int min_idx)) in
       if sample_width > Int64.of_int stat_max_lines
-      then 1+stat_max_lines,
+      then stat_max_lines,
         int_of_float (ceil (Int64.to_float sample_width /. float_of_int stat_max_lines))
-      else 1+max_idx-min_idx, 1
+      else max_idx-min_idx, 1
     in
+    let hist_size = if min_idx + bucket_size * hist_size <= max_idx then 1+hist_size else hist_size in
     (* accumulate bucket counts *)
     let max_val = ref 0 in (* max value after grouping by buckets *)
     let bucket_count = Array.init hist_size (fun _ -> 0) in
@@ -1473,16 +1603,23 @@ module Test = struct
     Printf.bprintf out
       "  num: %d, avg: %.2f, stddev: %.2f, median %d, min %d, max %d\n"
       !num !avg stddev !median min_idx max_idx;
+    let indwidth =
+      max (String.length (Printf.sprintf "%d" min_idx))
+        (max (String.length (Printf.sprintf "%d" max_idx))
+           (String.length (Printf.sprintf "%d" (min_idx + bucket_size * hist_size)))) in
+    let labwidth = if bucket_size=1 then indwidth else 2+2*indwidth in
     for i = 0 to hist_size - 1 do
       let i' = min_idx + i * bucket_size in
       let blabel =
         if bucket_size=1
-        then Printf.sprintf "%d" i'
-        else Printf.sprintf "%d..%d" i' (i'+bucket_size-1) in
+        then Printf.sprintf "%*d" indwidth i'
+        else
+          let bucket_bound = i'+bucket_size-1 in
+          Printf.sprintf "%*d..%*d" indwidth i' indwidth (if bucket_bound < i' then max_int else bucket_bound) in
       let bcount = bucket_count.(i) in
       (* NOTE: keep in sync *)
       let bar_len = bcount * 55 / !max_val in
-      Printf.bprintf out "  %15s: %-56s %10d\n" blabel (String.make bar_len '#') bcount
+      Printf.bprintf out "  %*s: %-56s %10d\n" labwidth blabel (String.make bar_len '#') bcount
     done;
     Buffer.contents out
 
@@ -1495,16 +1632,21 @@ module Test = struct
   let print_fail arb name l =
     print_test_fail name (List.map (print_c_ex arb) l)
 
+  let print_fail_other name ~msg =
+    print_test_fail name [msg]
+
   let print_error ?(st="") arb name (i,e) =
     print_test_error name (print_c_ex arb i) e st
 
   let check_result cell res = match res.R.state with
     | R.Success -> ()
-    | R.Error (i,e, bt) ->
-      raise (Test_error (cell.name, print_c_ex cell.arb i, e, bt))
-    | R.Failed l ->
-        let l = List.map (print_c_ex cell.arb) l in
-        raise (Test_fail (cell.name, l))
+    | R.Error {instance; exn; backtrace} ->
+      raise (Test_error (cell.name, print_c_ex cell.arb instance, exn, backtrace))
+    | R.Failed {instances=l} ->
+      let l = List.map (print_c_ex cell.arb) l in
+      raise (Test_fail (cell.name, l))
+    | R.Failed_other {msg} ->
+      raise (Test_fail (cell.name, [msg]))
 
   let check_cell_exn ?long ?call ?step ?rand cell =
     let res = check_cell ?long ?call ?step ?rand cell in
@@ -1524,11 +1666,14 @@ let find_example ?(name="<example>") ?count ~f g : _ Gen.t =
     let res = Test.check_cell ~rand:st cell in
     begin match res.TestResult.state with
       | TestResult.Success -> raise (No_example_found name)
-      | TestResult.Error (_, _, _) -> raise (No_example_found name)
-      | TestResult.Failed [] -> assert false
-      | TestResult.Failed (failed::_) ->
+      | TestResult.Error _ -> raise (No_example_found name)
+      | TestResult.Failed {instances=[]} -> assert false
+      | TestResult.Failed {instances=failed::_} ->
         (* found counter-example! *)
         failed.TestResult.instance
+      | TestResult.Failed_other {msg=_} ->
+        raise (No_example_found name)
+
     end
   in
   gen
