@@ -190,36 +190,74 @@ module Gen : sig
   val shuffle_l : 'a list -> 'a list t
   (** Creates a generator of shuffled lists. *)
 
-  val unit: unit t (** The unit generator. *)
+  val shuffle_w_l : (int * 'a) list -> 'a list t
+  (** Creates a generator of weighted shuffled lists. A given list is shuffled on each
+      generation according to the weights of its elements. An element with a larger weight
+      is more likely to be at the front of the list than an element with a smaller weight.
+      If we want to pick random elements from the (head of) list but need to prioritize
+      some elements over others, this generator can be useful.
 
-  val bool: bool t (** The boolean generator. *)
+      Example: given a weighted list [[1, "one"; 5, "five"; 10, "ten"]], the generator is
+      more likely to generate [["ten"; "five"; "one"]] or [["five"; "ten"; "one"]] than
+      [["one"; "ten"; "five"]] because "ten" and "five" have larger weights than "one".
 
-  val float: float t   (** Generates floating point numbers. *)
+      @since 0.11
+  *)
+
+  val unit : unit t (** The unit generator. *)
+
+  val bool : bool t (** The boolean generator. *)
+
+  val float : float t   (** Generates floating point numbers. *)
 
   val pfloat : float t (** Generates positive floating point numbers (0. included). *)
 
   val nfloat : float t (** Generates negative floating point numbers. (-0. included) *)
 
+  val float_bound_inclusive : float -> float t
+  (** [float_bound_inclusive bound] returns a random floating-point number between 0 and
+      [bound] (inclusive).  If [bound] is negative, the result is negative or zero.  If
+      [bound] is 0, the result is 0.
+      @since 0.11 *)
+
+  val float_bound_exclusive : float -> float t
+  (** [float_bound_exclusive bound] returns a random floating-point number between 0 and
+      [bound] (exclusive).  If [bound] is negative, the result is negative or zero.
+      @raise Invalid_argument if [bound] is zero.
+      @since 0.11 *)
+
+  val float_range : float -> float -> float t
+  (** [float_range low high] generates floating-point numbers within [low] and [high] (inclusive)
+      @raise Invalid_argument if [high < low] or if the range is larger than [max_float].
+      @since 0.11 *)
+
+  val (--.) : float -> float -> float t
+  (** Synonym for [float_range]
+      @since 0.11 *)
+
   val nat : int t (** Generates small natural numbers. *)
 
+  val big_nat : int t (** Generates natural numbers, possibly large. @since 0.10 *)
+
   val neg_int : int t (** Generates non-strictly negative integers (0 included). *)
 
   val pint : int t (** Generates non-strictly positive integers uniformly (0 included). *)
 
   val int : int t (** Generates integers uniformly. *)
 
-  val small_nat : int t (** Synonym to {!nat}. @since 0.5.1 *)
+  val small_nat : int t (** Small integers (< 100) @since 0.5.1 *)
 
   val small_int : int t
   (** Small UNSIGNED integers, for retrocompatibility.
       @deprecated use {!small_nat}. *)
 
   val small_signed_int : int t
-  (** Small SIGNED integers.
+  (** Small SIGNED integers, based on {!small_nat}.
       @since 0.5.2 *)
 
   val int_bound : int -> int t
   (** Uniform integer generator producing integers within [0... bound].
+      For [bound < 2^{30} - 1] uses [Random.State.int] for integer generation.
       @raise Invalid_argument if the argument is negative. *)
 
   val int_range : int -> int -> int t
@@ -283,16 +321,30 @@ module Gen : sig
 
   val string : ?gen:char t -> string t
   (** Builds a string generator. String size is generated by {!nat}.
-      Accepts an optional character generator (the default is {!char}). *)
+      Accepts an optional character generator (the default is {!char}).
+      See also {!string_of} and {!string_readable} for versions without
+      optional parameters. *)
+
+  val string_of : char t -> string t
+  (** Builds a string generator using the given character generator.
+      @since 0.11 *)
+
+  val string_readable : string t
+  (** Builds a string generator using the {!char} character generator.
+      @since 0.11 *)
 
   val small_string : ?gen:char t -> string t
-  (** Builds a string generator. String size is in the range [0--10].
+  (** Builds a string generator, length is {!small_nat}
       Accepts an optional character generator (the default is {!char}). *)
 
   val small_list : 'a t -> 'a list t
-  (** Generates lists of small size (range [0 -- 10]).
+  (** Generates lists of small size (see {!small_nat}).
       @since 0.5.3 *)
 
+  val small_array : 'a t -> 'a array t
+  (** Generates arrays of small size (see {!small_nat}).
+      @since 0.10 *)
+
   val join : 'a t t -> 'a t
   (** Collapses a generator of generators to simply a generator.
       @since 0.5 *)
@@ -307,8 +359,14 @@ module Gen : sig
       to the size-bounded generator.
       @since 0.5 *)
 
-  val fix : ('a sized -> 'a sized) -> 'a sized
-  (** Fixpoint combinator for generating recursive, size-bounded data types.
+  val fix : (('a -> 'b t) -> ('a -> 'b t)) -> 'a -> 'b t
+  (** Parametrized fixpoint combinator for generating recursive values.
+
+      The fixpoint is parametrized over an arbitrary state ('a), and the
+      fixpoint computation may change the value of this state in the recursive
+      calls.
+
+      In particular, this can be used for size-bounded generators ('a is int).
       The passed size-parameter should decrease to ensure termination. *)
 
   (** Example:
@@ -450,11 +508,19 @@ module Shrink : sig
       @since 0.7 *)
 
   val list : ?shrink:'a t -> 'a list t
-  (** Try to shrink lists by removing elements one by one.
+  (** Try to shrink lists by removing one or more elements.
       @param shrink if provided, will be used to also try to reduce
       the elements of the list themselves (e.g. in an [int list]
       one can try to decrease the integers). *)
 
+  val list_spine : 'a list t
+  (** Try to shrink lists by removing one or more elements.
+      @since 0.10 *)
+
+  val list_elems : 'a t -> 'a list t
+  (** Shrinks the elements of a list, without changing the list size.
+      @since 0.10 *)
+
   val array : ?shrink:'a t -> 'a array t
   (** Shrink an array.
       @param shrink see {!list} *)
@@ -605,19 +671,28 @@ module TestResult : sig
 
   type 'a failed_state = 'a counter_ex list
 
+  (** Result state.
+      changed in 0.10 (move to inline records, add Fail_other) *)
   type 'a state =
     | Success
-    | Failed of 'a failed_state (** Failed instances *)
-    | Error of 'a counter_ex * exn * string (** Error, backtrace, and instance
-                                                that triggered it *)
+    | Failed of {
+        instances: 'a failed_state; (** Failed instance(s) *)
+      }
+    | Failed_other of {msg: string}
+    | Error of {
+        instance: 'a counter_ex;
+        exn: exn;
+        backtrace: string;
+      } (** Error, backtrace, and instance that triggered it *)
 
   (* result returned by running a test *)
-  type 'a t = {
+  type 'a t = private {
     mutable state : 'a state;
     mutable count: int;  (* Number of tests *)
     mutable count_gen: int; (* Number of generated cases *)
     collect_tbl: (string, int) Hashtbl.t lazy_t;
     stats_tbl: ('a stat * (int, int) Hashtbl.t) list; (** @since 0.6 *)
+    mutable warnings: string list;
     mutable instances: 'a list;
     (** List of instances used for this test, in no particular order.
         @since 0.9 *)
@@ -631,6 +706,10 @@ module TestResult : sig
   (** Obtain statistics
       @since 0.6 *)
 
+  val warnings : _ t -> string list
+  (** Obtain list of warnings
+      @since 0.10 *)
+
   val is_success : _ t -> bool
   (** Returns true iff the state if [Success]
       @since 0.9 *)
@@ -650,6 +729,7 @@ module Test : sig
       @since 0.7 *)
 
   val make_cell :
+    ?if_assumptions_fail:([`Fatal | `Warning] * float) ->
     ?count:int -> ?long_factor:int -> ?max_gen:int -> ?max_fail:int ->
     ?small:('a -> int) -> ?name:string -> 'a arbitrary -> ('a -> bool) ->
     'a cell
@@ -669,6 +749,13 @@ module Test : sig
         the field [arbitrary.small].
         If there is no shrinking function but there is a [small]
         function, only the smallest failures will be printed.
+      @param if_assumptions_fail the minimum
+        fraction of tests that must satisfy the precondition for a success
+        to be considered valid.
+        The fraction should be between 0. and 1.
+        A warning will be emitted otherwise if
+        the flag is [`Warning], the test will be a failure if the flag is [`Fatal].
+        (since 0.10)
   *)
 
   val get_arbitrary : 'a cell -> 'a arbitrary
@@ -689,6 +776,7 @@ module Test : sig
       put tests on different types in the same list of tests. *)
 
   val make :
+    ?if_assumptions_fail:([`Fatal | `Warning] * float) ->
     ?count:int -> ?long_factor:int -> ?max_gen:int -> ?max_fail:int ->
     ?small:('a -> int) -> ?name:string -> 'a arbitrary -> ('a -> bool) -> t
   (** [make arb prop] builds a test that checks property [prop] on instances
@@ -712,6 +800,7 @@ module Test : sig
   val print_instance : 'a arbitrary -> 'a -> string
   val print_c_ex : 'a arbitrary -> 'a TestResult.counter_ex -> string
   val print_fail : 'a arbitrary -> string -> 'a TestResult.counter_ex list -> string
+  val print_fail_other : string -> msg:string -> string
   val print_error : ?st:string -> 'a arbitrary -> string -> 'a TestResult.counter_ex * exn -> string
   val print_test_fail : string -> string list -> string
   val print_test_error : string -> string -> exn -> string -> string
@@ -849,6 +938,22 @@ val pos_float : float arbitrary
 val neg_float : float arbitrary
 (** Negative float generator (no nan and no infinities). *)
 
+val float_bound_inclusive : float -> float arbitrary
+(** [float_bound_inclusive n] is uniform between [0] and [n] included. If [bound] is
+    negative, the result is negative or zero.  If [bound] is 0, the result is 0.
+    @since 0.11 *)
+
+val float_bound_exclusive : float -> float arbitrary
+(** [float_bound_exclusive n] is uniform between [0] included and [n] excluded.
+    If [bound] is negative, the result is negative or zero.
+    @raise Invalid_argument if [bound] is zero.
+    @since 0.11 *)
+
+val float_range : float -> float -> float arbitrary
+(** [float_range low high] is uniform between [low] included and [high] included.
+    @raise Invalid_argument if [low > high] or if the range is larger than [max_float].
+    @since 0.11 *)
+
 val int : int arbitrary
 (** Int generator. Uniformly distributed. *)
 
@@ -915,10 +1020,10 @@ val string : string arbitrary
     and distribution of characters of [char]. *)
 
 val small_string : string arbitrary
-(** Same as {!string} but with a small length (that is, [0--10]). *)
+(** Same as {!string} but with a small length (ie {!Gen.small_nat} ). *)
 
 val small_list : 'a arbitrary -> 'a list arbitrary
-(** Generates lists of small size (range [0 -- 10]).
+(** Generates lists of small size (see {!Gen.small_nat}).
     @since 0.5.3 *)
 
 val string_of_size : int Gen.t -> string arbitrary
